@@ -81,6 +81,10 @@ sub call {
 	$req->header('Content-Type'   => 'text/xml');
 	$req->header('User-Agent'     => $self->{ua});
 	$req->header( $_ => $args{headers}{$_} ) for keys %{$args{headers}};
+	if( utf8::is_utf8($args{body}) ) {
+		carp "got an utf8 body: $args{body}";
+		utf8::encode($args{body});
+	}
 	{
 		use bytes;
 		$req->header( 'Content-Length' => length($args{body}) );
